"""Initialize an etherpump folder"""

import json
import os
import sys
from argparse import ArgumentParser
from urllib.parse import urlencode, urlparse, urlunparse
from urllib.request import HTTPError, URLError, urlopen


def get_api(url, cmd=None, data=None, verbose=False):
    try:
        useurl = url + cmd
        if data:
            useurl += "?" + urlencode(data)
        if verbose:
            print("trying", useurl, file=sys.stderr)
        resp = urlopen(useurl).read()
        resp = resp.decode("utf-8")
        resp = json.loads(resp)
        if "code" in resp and "message" in resp:
            return resp
    except ValueError as e:
        if verbose:
            print("  ValueError", e, file=sys.stderr)
        return
    except HTTPError as e:
        if verbose:
            print("  HTTPError", e, file=sys.stderr)
        if e.code == 401:
            # Unauthorized is how the API responds to an incorrect API key
            return {"code": 401, "message": e}


def tryapiurl(url, verbose=False):
    """
    Try to use url as api, correcting if possible.
    Returns corrected / normalized URL, or None if not possible
    """
    try:
        scheme, netloc, path, params, query, fragment = urlparse(url)
        if scheme == "":
            url = "http://" + url
        scheme, netloc, path, params, query, fragment = urlparse(url)
        params, query, fragment = ("", "", "")
        path = path.strip("/")
        # 1. try directly...
        apiurl = (
            urlunparse((scheme, netloc, path, params, query, fragment)) + "/"
        )
        if get_api(apiurl, "listAllPads", verbose=verbose):
            return apiurl
        # 2. try with += api/1.2.9
        path = os.path.join(path, "api", "1.2.9") + "/"
        apiurl = urlunparse((scheme, netloc, path, params, query, fragment))
        if get_api(apiurl, "listAllPads", verbose=verbose):
            return apiurl
    except URLError as e:
        print("URLError", e, file=sys.stderr)


def main(args):
    p = ArgumentParser("initialize an etherpump folder")
    p.add_argument(
        "arg",
        nargs="*",
        default=[],
        help="optional positional args: path etherpadurl",
    )
    p.add_argument("--path", default=None, help="path to initialize")
    p.add_argument("--padurl", default=None, help="")
    p.add_argument("--apikey", default=None, help="")
    p.add_argument("--verbose", default=False, action="store_true", help="")
    p.add_argument("--reinit", default=False, action="store_true", help="")
    args = p.parse_args(args)

    path = args.path
    if path == None and len(args.arg):
        path = args.arg[0]
    if not path:
        path = "."

    edpath = os.path.join(path, ".etherpump")
    try:
        os.makedirs(edpath)
    except OSError:
        pass

    padinfo = {}
    padinfopath = os.path.join(edpath, "settings.json")
    try:
        with open(padinfopath) as f:
            padinfo = json.load(f)
        if not args.reinit:
            print("Folder already initialized. Use --reinit to reset settings")
            sys.exit(0)
    except IOError:
        pass
    except ValueError:
        # bad json file, reset it
        pass

    apiurl = args.padurl
    while True:
        if apiurl:
            apiurl = tryapiurl(apiurl, verbose=args.verbose)
        if apiurl:
            # print ("Got APIURL: {0}".format(apiurl))
            break
        apiurl = input(
            "Please type the URL of the etherpad (e.g. https://pad.vvvvvvaria.org): "
        ).strip()
    padinfo["apiurl"] = apiurl
    apikey = args.apikey
    while True:
        if apikey:
            resp = get_api(
                apiurl, "listAllPads", {"apikey": apikey}, verbose=args.verbose
            )
            if resp and resp["code"] == 0:
                # print ("GOOD")
                break
            else:
                print("bad")
        print(
            "The APIKEY is the contents of the file APIKEY.txt in the etherpad folder",
            file=sys.stderr,
        )
        apikey = input("Please paste the APIKEY: ").strip()
    padinfo["apikey"] = apikey

    with open(padinfopath, "w") as f:
        json.dump(padinfo, f, indent=2)



"""Call listAllPads and print the results"""

import json
import sys
from argparse import ArgumentParser
from urllib.parse import urlencode, urlparse, urlunparse
from urllib.request import HTTPError, URLError, urlopen

from etherpump.commands.common import getjson


def main(args):
    p = ArgumentParser("call listAllPads and print the results")
    p.add_argument(
        "--padinfo",
        default=".etherpump/settings.json",
        help="settings, default: .etherdump/settings.json",
    )
    p.add_argument("--showurl", default=False, action="store_true")
    p.add_argument(
        "--format",
        default="lines",
        help="output format: lines, json; default lines",
    )
    args = p.parse_args(args)

    with open(args.padinfo) as f:
        info = json.load(f)
    apiurl = info.get("apiurl")
    # apiurl = {0[protocol]}://{0[hostname]}:{0[port]}{0[apiurl]}{0[apiversion]}/".format(info)
    data = {}
    data["apikey"] = info["apikey"]
    requesturl = apiurl + "listAllPads?" + urlencode(data)
    if args.showurl:
        print(requesturl)
    else:
        results = getjson(requesturl)["data"]["padIDs"]
        if args.format == "json":
            print(json.dumps(results))
        else:
            for r in results:
                print(r)



from functools import wraps
from os.path import exists
from pathlib import Path
from urllib.parse import urlencode

from etherpump.commands.common import getjson, loadpadinfo
from etherpump.commands.creatediffhtml import main as creatediffhtml  # noqa
from etherpump.commands.deletepad import main as deletepad  # noqa
from etherpump.commands.dumpcsv import main as dumpcsv  # noqa
from etherpump.commands.gethtml import main as gethtml  # noqa
from etherpump.commands.gettext import main as gettext  # noqa
from etherpump.commands.index import main as index  # noqa
from etherpump.commands.init import main  # noqa
from etherpump.commands.init import main as init
from etherpump.commands.list import main as list  # noqa
from etherpump.commands.listauthors import main as listauthors  # noqa
from etherpump.commands.publication import main as publication  # noqa
from etherpump.commands.pull import main as pull
from etherpump.commands.revisionscount import main as revisionscount  # noqa
from etherpump.commands.sethtml import main as sethtml  # noqa
from etherpump.commands.settext import main as settext  # noqa
from etherpump.commands.showmeta import main as showmeta  # noqa


def ensure_init():
    path = Path(".etherpump/settings.json").absolute()
    if not exists(path):
        try:
            main([])
        except SystemExit:
            pass


def get_pad_ids():
    info = loadpadinfo(Path(".etherpump/settings.json"))
    data = {"apikey": info["apikey"]}
    url = info["localapiurl"] + "listAllPads?" + urlencode(data)
    return getjson(url)["data"]["padIDs"]


def magic_word(word, fresh=True):
    ensure_init()

    if fresh:
        pull(["--text", "--meta", "--publish-opt-in", "--publish", word])

    pads = {}
    pad_ids = get_pad_ids()
    for pad_id in pad_ids:
        path = Path("./p/{}.raw.txt".format(pad_id)).absolute()
        try:
            with open(path, "r") as handle:
                text = handle.read()
            if word in text:
                pads[pad_id] = {}
                pads[pad_id]["txt"] = text
        except FileNotFoundError:
            continue

    def _magic_word(func):
        @wraps(func)
        def wrapper(*args, **kwargs):
            return func(pads)

        return wrapper

    return _magic_word



"""Calls the createDiffHTML API function for the given padid"""

import json
from argparse import ArgumentParser
from urllib.error import HTTPError, URLError
from urllib.parse import urlencode
from urllib.request import urlopen


def main(args):
    p = ArgumentParser(
        "calls the createDiffHTML API function for the given padid"
    )
    p.add_argument("padid", help="the padid")
    p.add_argument(
        "--padinfo",
        default=".etherpump/settings.json",
        help="settings, default: .etherdump/settings.json",
    )
    p.add_argument("--showurl", default=False, action="store_true")
    p.add_argument(
        "--format",
        default="text",
        help="output format, can be: text, json; default: text",
    )
    p.add_argument(
        "--rev", type=int, default=None, help="revision, default: latest"
    )
    args = p.parse_args(args)

    with open(args.padinfo) as f:
        info = json.load(f)
    apiurl = info.get("apiurl")
    # apiurl = "{0[protocol]}://{0[hostname]}:{0[port]}{0[apiurl]}{0[apiversion]}/".format(info)
    data = {}
    data["apikey"] = info["apikey"]
    data["padID"] = args.padid
    data["startRev"] = "0"
    if args.rev != None:
        data["rev"] = args.rev
    requesturl = apiurl + "createDiffHTML?" + urlencode(data)
    if args.showurl:
        print(requesturl)
    else:
        try:
            results = json.load(urlopen(requesturl))["data"]
            if args.format == "json":
                print(json.dumps(results))
            else:
                print(results["html"])
        except HTTPError as e:
            pass



"""Generate pages from etherpumps using a template"""
import json
import os
import re
import sys
import time
from argparse import ArgumentParser
from datetime import datetime
from urllib.parse import urlparse, urlunparse

import dateutil.parser
from jinja2 import Environment, FileSystemLoader

from etherpump.commands.common import *  # noqa

"""
index:
    Generate pages from etherpumps using a template.

    Built-in templates: rss.xml, index.html

"""


def group(items, key=lambda x: x):
    """ returns a list of lists, of items grouped by a key function """
    ret = []
    keys = {}
    for item in items:
        k = key(item)
        if k not in keys:
            keys[k] = []
        keys[k].append(item)
    for k in sorted(keys):
        keys[k].sort()
        ret.append(keys[k])
    return ret


def splitextlong(x):
    """ split "long" extensions, i.e. foo.bar.baz => ('foo', '.bar.baz') """
    m = re.search(r"^(.*?)(\..*)$", x)
    if m:
        return m.groups()
    else:
        return x, ""


def base(x):
    return splitextlong(x)[0]


def excerpt(t, chars=25):
    if len(t) > chars:
        t = t[:chars] + "..."
    return t


def absurl(url, base=None):
    if not url.startswith("http"):
        return base + url
    return url


def url_base(url):
    (scheme, netloc, path, params, query, fragment) = urlparse(url)
    path, _ = os.path.split(path.lstrip("/"))
    ret = urlunparse((scheme, netloc, path, None, None, None))
    if ret:
        ret += "/"
    return ret


def datetimeformat(t, format="%Y-%m-%d %H:%M:%S"):
    if type(t) == str:
        dt = dateutil.parser.parse(t)
        return dt.strftime(format)
    else:
        return time.strftime(format, time.localtime(t))


def main(args):
    p = ArgumentParser("Convert dumped files to a document via a template.")

    p.add_argument("input", nargs="+", help="Files to list (.meta.json files)")

    p.add_argument(
        "--templatepath",
        default=None,
        help="path to find templates, default: built-in",
    )
    p.add_argument(
        "--template",
        default="index.html",
        help="template name, built-ins include index.html, rss.xml; default: index.html",
    )
    p.add_argument(
        "--padinfo",
        default=".etherpump/settings.json",
        help="settings, default: ./.etherdump/settings.json",
    )
    # p.add_argument("--zerorevs", default=False, action="store_true", help="include pads with zero revisions, default: False (i.e. pads with no revisions are skipped)")

    p.add_argument(
        "--order",
        default="padid",
        help="order, possible values: padid, pad (no group name), lastedited, (number of) authors, revisions, default: padid",
    )
    p.add_argument(
        "--reverse",
        default=False,
        action="store_true",
        help="reverse order, default: False (reverse chrono)",
    )
    p.add_argument(
        "--limit",
        type=int,
        default=0,
        help="limit to number of items, default: 0 (no limit)",
    )
    p.add_argument(
        "--skip",
        default=None,
        type=int,
        help="skip this many items, default: None",
    )

    p.add_argument(
        "--content",
        default=False,
        action="store_true",
        help="rss: include (full) content tag, default: False",
    )
    p.add_argument(
        "--link",
        default="diffhtml,html,text",
        help="link variable will be to this version, can be comma-delim list, use first avail, default: diffhtml,html,text",
    )
    p.add_argument(
        "--linkbase",
        default=None,
        help="base url to use for links, default: try to use the feedurl",
    )
    p.add_argument("--output", default=None, help="output, default: stdout")

    p.add_argument(
        "--files",
        default=False,
        action="store_true",
        help="include files (experimental)",
    )

    pg = p.add_argument_group("template variables")
    pg.add_argument(
        "--feedurl",
        default="feed.xml",
        help="rss: to use as feeds own (self) link, default: feed.xml",
    )
    pg.add_argument(
        "--siteurl",
        default=None,
        help="rss: to use as channel's site link, default: the etherpad url",
    )
    pg.add_argument(
        "--title",
        default="etherpump",
        help="title for document or rss feed channel title, default: etherdump",
    )
    pg.add_argument(
        "--description",
        default="",
        help="rss: channel description, default: empty",
    )
    pg.add_argument(
        "--language", default="en-US", help="rss: feed language, default: en-US"
    )
    pg.add_argument(
        "--updatePeriod",
        default="daily",
        help="rss: updatePeriod, possible values: hourly, daily, weekly, monthly, yearly; default: daily",
    )
    pg.add_argument(
        "--updateFrequency",
        default=1,
        type=int,
        help="rss: update frequency within the update period (where 2 would mean twice per period); default: 1",
    )
    pg.add_argument(
        "--generator",
        default="https://gitlab.com/activearchives/etherpump",
        help="generator, default: https://gitlab.com/activearchives/etherdump",
    )
    pg.add_argument(
        "--timestamp",
        default=None,
        help="timestamp, default: now (e.g. 2015-12-01 12:30:00)",
    )
    pg.add_argument("--next", default=None, help="next link, default: None)")
    pg.add_argument("--prev", default=None, help="prev link, default: None")

    args = p.parse_args(args)

    tmpath = args.templatepath
    # Default path for template is the built-in data/templates
    if tmpath == None:
        tmpath = os.path.split(os.path.abspath(__file__))[0]
        tmpath = os.path.split(tmpath)[0]
        tmpath = os.path.join(tmpath, "data", "templates")

    env = Environment(loader=FileSystemLoader(tmpath))
    env.filters["excerpt"] = excerpt
    env.filters["datetimeformat"] = datetimeformat
    template = env.get_template(args.template)

    info = loadpadinfo(args.padinfo)

    inputs = args.input
    inputs.sort()
    # Use "base" to strip (longest) extensions
    # inputs = group(inputs, base)

    def wrappath(p):
        path = "./{0}".format(p)
        ext = os.path.splitext(p)[1][1:]
        return {"url": path, "path": path, "code": 200, "type": ext}

    def metaforpaths(paths):
        ret = {}
        pid = base(paths[0])
        ret["pad"] = ret["padid"] = pid
        ret["versions"] = [wrappath(x) for x in paths]
        lastedited = None
        for p in paths:
            mtime = os.stat(p).st_mtime
            if lastedited == None or mtime > lastedited:
                lastedited = mtime
        ret["lastedited_iso"] = datetime.fromtimestamp(lastedited).strftime(
            "%Y-%m-%dT%H:%M:%S"
        )
        ret["lastedited_raw"] = mtime
        return ret

    def loadmeta(p):
        # Consider a set of grouped files
        # Otherwise, create a "dummy" one that wraps all the files as versions
        if p.endswith(".meta.json"):
            with open(p) as f:
                return json.load(f)
        # # IF there is a .meta.json, load it & MERGE with other files
        # if ret:
        #     # TODO: merge with other files
        #     for p in paths:
        #         if "./"+p not in ret['versions']:
        #             ret['versions'].append(wrappath(p))
        #     return ret
        # else:
        #     return metaforpaths(paths)

    def fixdates(padmeta):
        d = dateutil.parser.parse(padmeta["lastedited_iso"])
        padmeta["lastedited"] = d
        padmeta["lastedited_822"] = d.strftime("%a, %d %b %Y %H:%M:%S +0000")
        return padmeta

    pads = list(map(loadmeta, inputs))
    pads = [x for x in pads if x != None]
    pads = list(map(fixdates, pads))
    args.pads = list(pads)

    def could_have_base(x, y):
        return x == y or (x.startswith(y) and x[len(y) :].startswith("."))

    def get_best_pad(x):
        for pb in padbases:
            p = pads_by_base[pb]
            if could_have_base(x, pb):
                return p

    def has_version(padinfo, path):
        return [
            x
            for x in padinfo["versions"]
            if "path" in x and x["path"] == "./" + path
        ]

    if args.files:
        inputs = args.input
        inputs.sort()
        removelist = []

        pads_by_base = {}
        for p in args.pads:
            # print ("Trying padid", p['padid'], file=sys.stderr)
            padbase = os.path.splitext(p["padid"])[0]
            pads_by_base[padbase] = p
        padbases = list(pads_by_base.keys())
        # SORT THEM LONGEST FIRST TO ensure that LONGEST MATCHES MATCH
        padbases.sort(key=lambda x: len(x), reverse=True)
        # print ("PADBASES", file=sys.stderr)
        # for pb in padbases:
        #     print ("  ", pb, file=sys.stderr)
        print("pairing input files with pads", file=sys.stderr)
        for x in inputs:
            # pair input with a pad if possible
            xbasename = os.path.basename(x)
            p = get_best_pad(xbasename)
            if p:
                if not has_version(p, x):
                    print(
                        "Grouping file {0} with pad {1}".format(x, p["padid"]),
                        file=sys.stderr,
                    )
                    p["versions"].append(wrappath(x))
                else:
                    print(
                        "Skipping existing version {0} ({1})...".format(
                            x, p["padid"]
                        ),
                        file=sys.stderr,
                    )
                removelist.append(x)
        # Removed Matches files
        for x in removelist:
            inputs.remove(x)
        print("Remaining files:", file=sys.stderr)
        for x in inputs:
            print(x, file=sys.stderr)
        print(file=sys.stderr)
        # Add "fake" pads for remaining files
        for x in inputs:
            args.pads.append(metaforpaths([x]))

    if args.timestamp == None:
        args.timestamp = datetime.now().strftime("%Y-%m-%d %H:%M:%S")

    padurlbase = re.sub(r"api/1.2.9/$", "p/", info["apiurl"])
    args.siteurl = args.siteurl or padurlbase
    args.utcnow = datetime.utcnow().strftime("%a, %d %b %Y %H:%M:%S +0000")

    # order items & apply limit
    if args.order == "lastedited":
        args.pads.sort(
            key=lambda x: x.get("lastedited_iso"), reverse=args.reverse
        )
    elif args.order == "pad":
        args.pads.sort(key=lambda x: x.get("pad"), reverse=args.reverse)
    elif args.order == "padid":
        args.pads.sort(key=lambda x: x.get("padid"), reverse=args.reverse)
    elif args.order == "revisions":
        args.pads.sort(key=lambda x: x.get("revisions"), reverse=args.reverse)
    elif args.order == "authors":
        args.pads.sort(
            key=lambda x: len(x.get("authors")), reverse=args.reverse
        )
    else:
        raise Exception("That ordering is not implemented!")

    if args.limit:
        args.pads = args.pads[: args.limit]

    # add versions_by_type, add in full text
    # add link (based on args.link)
    linkversions = args.link.split(",")
    linkbase = args.linkbase or url_base(args.feedurl)
    # print ("linkbase", linkbase, args.linkbase, args.feedurl)

    for p in args.pads:
        versions_by_type = {}
        p["versions_by_type"] = versions_by_type
        for v in p["versions"]:
            t = v["type"]
            versions_by_type[t] = v

        if "text" in versions_by_type:
            try:
                with open(versions_by_type["text"]["path"]) as f:
                    p["text"] = f.read()
            except FileNotFoundError:
                p["text"] = ""
        # ADD IN LINK TO PAD AS "link"
        for v in linkversions:
            if v in versions_by_type:
                vdata = versions_by_type[v]
                try:
                    if v == "pad" or os.path.exists(vdata["path"]):
                        p["link"] = absurl(vdata["url"], linkbase)
                        break
                except KeyError as e:
                    pass

    if args.output:
        with open(args.output, "w") as f:
            print(template.render(vars(args)), file=f)
    else:
        print(template.render(vars(args)))



#!/usr/bin/env python3
import os
import sys

DATAPATH = os.path.join(os.path.dirname(os.path.realpath(__file__)), "data")

__VERSION__ = "0.0.20"


def subcommands():
    """List all sub-commands for the `--help` output."""
    output = []

    subcommands = [
        "creatediffhtml",
        "deletepad",
        "dumpcsv",
        "gethtml",
        "gettext",
        "index",
        "init",
        "list",
        "listauthors",
        "publication",
        "pull",
        "revisionscount",
        "sethtml",
        "settext",
        "showmeta",
    ]

    for subcommand in subcommands:
        try:
            # http://stackoverflow.com/questions/301134/dynamic-module-import-in-python
            doc = __import__(
                "etherpump.commands.%s" % subcommand,
                fromlist=["etherdump.commands"],
            ).__doc__
        except ModuleNotFoundError:
            doc = ""
        output.append(f"    {subcommand}: {doc}")

    output.sort()

    return "\n".join(output)


usage = """
         _
        | |
  _ _|_ | |     _   ,_     _          _  _  _     _
 |/  |  |/ \   |/  /  |  |/ \_|   |  / |/ |/ |  |/ \_
 |__/|_/|   |_/|__/   |_/|__/  \_/|_/  |  |  |_/|__/
                        /|                     /|
                        \|                     \|
Usage:
    etherpump CMD

where CMD could be:
{}

For more information on each command try:
    etherpump CMD --help""".format(
    subcommands()
)


def main():
    try:
        cmd = sys.argv[1]
        if cmd.startswith("-"):
            args = sys.argv
        else:
            args = sys.argv[2:]

        if len(sys.argv) < 3:
            if any(arg in args for arg in ["--help", "-h"]):
                print(usage)
                sys.exit(0)
            elif any(arg in args for arg in ["--version", "-v"]):
                print("etherpump {}".format(__VERSION__))
                sys.exit(0)

    except IndexError:
        print(usage)
        sys.exit(0)

    try:
        # http://stackoverflow.com/questions/301134/dynamic-module-import-in-python
        cmdmod = __import__(
            "etherpump.commands.%s" % cmd, fromlist=["etherdump.commands"]
        )
        cmdmod.main(args)
    except ImportError as e:
        print(
            "Error performing command '{0}'\n(python said: {1})\n".format(
                cmd, e
            )
        )
        print(usage)



"""Update meta data files for those that have changed"""
import os
from argparse import ArgumentParser
from urllib.parse import urlencode

from .common import *  # noqa

"""
status (meta):
    Update meta data files for those that have changed.
    Check for changed pads by looking at revisions & comparing to existing


design decisions...
ok based on the fact that only the txt file is pushable (via setText)
it makes sense to give this file "primacy" ... ie to put the other forms
(html, diff.html) in a special place (if created at all). Otherwise this
complicates the "syncing" idea....

"""


class PadItemException(Exception):
    pass


class PadItem:
    def __init__(self, padid=None, path=None, padexists=False):
        self.padexists = padexists
        if padid and path:
            raise PadItemException("only give padid or path")
        if not (padid or path):
            raise PadItemException("either padid or path must be specified")
        if padid:
            self.padid = padid
            self.path = padpath(padid, group_path="g")
        else:
            self.path = path
            self.padid = padpath2id(path)

    @property
    def status(self):
        if self.fileexists:
            if self.padexists:
                return "S"
            else:
                return "F"
        elif self.padexists:
            return "P"
        else:
            return "?"

    @property
    def fileexists(self):
        return os.path.exists(self.path)


def ignore_p(path, settings=None):
    if path.startswith("."):
        return True


def main(args):
    p = ArgumentParser(
        "Check for pads that have changed since last sync (according to .meta.json)"
    )
    # p.add_argument("padid", nargs="*", default=[])
    p.add_argument(
        "--padinfo",
        default=".etherpump/settings.json",
        help="settings, default: .etherdump/settings.json",
    )
    p.add_argument(
        "--zerorevs",
        default=False,
        action="store_true",
        help="include pads with zero revisions, default: False (i.e. pads with no revisions are skipped)",
    )
    p.add_argument(
        "--pub",
        default=".",
        help="folder to store files for public pads, default: pub",
    )
    p.add_argument(
        "--group",
        default="g",
        help="folder to store files for group pads, default: g",
    )
    p.add_argument(
        "--skip",
        default=None,
        type=int,
        help="skip this many items, default: None",
    )
    p.add_argument(
        "--meta",
        default=False,
        action="store_true",
        help="download meta to PADID.meta.json, default: False",
    )
    p.add_argument(
        "--text",
        default=False,
        action="store_true",
        help="download text to PADID.txt, default: False",
    )
    p.add_argument(
        "--html",
        default=False,
        action="store_true",
        help="download html to PADID.html, default: False",
    )
    p.add_argument(
        "--dhtml",
        default=False,
        action="store_true",
        help="download dhtml to PADID.dhtml, default: False",
    )
    p.add_argument(
        "--all",
        default=False,
        action="store_true",
        help="download all files (meta, text, html, dhtml), default: False",
    )
    args = p.parse_args(args)

    info = loadpadinfo(args.padinfo)
    data = {}
    data["apikey"] = info["apikey"]

    padsbypath = {}

    # listAllPads
    padids = getjson(info["apiurl"] + "listAllPads?" + urlencode(data))["data"][
        "padIDs"
    ]
    padids.sort()
    for padid in padids:
        pad = PadItem(padid=padid, padexists=True)
        padsbypath[pad.path] = pad

    files = os.listdir(args.pub)
    files = [x for x in files if not ignore_p(x)]
    files.sort()
    for p in files:
        pad = padsbypath.get(p)
        if not pad:
            pad = PadItem(path=p)
            padsbypath[pad.path] = pad

    pads = list(padsbypath.values())
    pads.sort(key=lambda x: (x.status, x.padid))

    curstat = None
    for p in pads:
        if p.status != curstat:
            curstat = p.status
            if curstat == "F":
                print("New/changed files")
            elif curstat == "P":
                print("New/changed pads")
            elif curstat == ".":
                print("Up to date")
        print("   ", p.status, p.padid)



#!/usr/bin/env python

import json
from argparse import ArgumentParser


def main(args):
    p = ArgumentParser("")
    p.add_argument("input", nargs="+", help="filenames")
    p.add_argument("--indent", type=int, default=2, help="indent")
    args = p.parse_args(args)
    inputs = args.input
    inputs.sort()
    ret = []
    for p in inputs:
        with open(p) as f:
            meta = json.load(f)
            ret.append(meta)

    if args.indent:
        print(json.dumps(ret, indent=args.indent))
    else:
        print(json.dumps(ret))



#!/usr/bin/env python3


import os
import sys
from argparse import ArgumentParser
from xml.etree import ElementTree as ET

from html5lib import parse


def etree_indent(elem, level=0):
    i = "\n" + level * "  "
    if len(elem):
        if not elem.text or not elem.text.strip():
            elem.text = i + "  "
        if not elem.tail or not elem.tail.strip():
            elem.tail = i
        for elem in elem:
            etree_indent(elem, level + 1)
        if not elem.tail or not elem.tail.strip():
            elem.tail = i
    else:
        if level and (not elem.tail or not elem.tail.strip()):
            elem.tail = i


def get_link_type(url):
    lurl = url.lower()
    if lurl.endswith(".html") or lurl.endswith(".htm"):
        return "text/html"
    elif lurl.endswith(".txt"):
        return "text/plain"
    elif lurl.endswith(".rss"):
        return "application/rss+xml"
    elif lurl.endswith(".atom"):
        return "application/atom+xml"
    elif lurl.endswith(".json"):
        return "application/json"
    elif lurl.endswith(".js") or lurl.endswith(".jsonp"):
        return "text/javascript"


def pluralize(x):
    if type(x) == list or type(x) == tuple:
        return x
    else:
        return (x,)


def html5tidy(
    doc, charset="utf-8", title=None, scripts=None, links=None, indent=False
):
    if scripts:
        script_srcs = [x.attrib.get("src") for x in doc.findall(".//script")]
        for src in pluralize(scripts):
            if src not in script_srcs:
                script = ET.SubElement(doc.find(".//head"), "script", src=src)
                script_srcs.append(src)

    if links:
        existinglinks = {}
        for elt in doc.findall(".//link"):
            href = elt.attrib.get("href")
            if href:
                existinglinks[href] = elt
        for link in links:
            linktype = link.get("type") or get_link_type(link["href"])
            if link["href"] in existinglinks:
                elt = existinglinks[link["href"]]
                elt.attrib["rel"] = link["rel"]
            else:
                elt = ET.SubElement(
                    doc.find(".//head"),
                    "link",
                    href=link["href"],
                    rel=link["rel"],
                )
            if linktype:
                elt.attrib["type"] = linktype
            if "title" in link:
                elt.attrib["title"] = link["title"]

    if charset:
        meta_charsets = [
            x.attrib.get("charset")
            for x in doc.findall(".//meta")
            if x.attrib.get("charset") != None
        ]
        if not meta_charsets:
            meta = ET.SubElement(doc.find(".//head"), "meta", charset=charset)

    if title != None:
        titleelt = doc.find(".//title")
        if not titleelt:
            titleelt = ET.SubElement(doc.find(".//head"), "title")
        titleelt.text = title

    if indent:
        etree_indent(doc)
    return doc


def main(args):
    p = ArgumentParser("")
    p.add_argument("input", nargs="?", default=None)
    p.add_argument("--indent", default=False, action="store_true")
    p.add_argument(
        "--mogrify",
        default=False,
        action="store_true",
        help="modify file in place",
    )
    p.add_argument(
        "--method",
        default="html",
        help="method, default: html, values: html, xml, text",
    )
    p.add_argument("--output", default=None, help="")
    p.add_argument("--title", default=None, help="ensure/add title tag in head")
    p.add_argument(
        "--charset", default="utf-8", help="ensure/add meta tag with charset"
    )
    p.add_argument(
        "--script", action="append", default=[], help="ensure/add script tag"
    )
    # <link>s, see https://www.w3.org/TR/html5/links.html#links
    p.add_argument(
        "--stylesheet",
        action="append",
        default=[],
        help="ensure/add style link",
    )
    p.add_argument(
        "--alternate",
        action="append",
        default=[],
        nargs="+",
        help="ensure/add alternate links (optionally followed by a title and type)",
    )
    p.add_argument(
        "--next",
        action="append",
        default=[],
        nargs="+",
        help="ensure/add alternate link",
    )
    p.add_argument(
        "--prev",
        action="append",
        default=[],
        nargs="+",
        help="ensure/add alternate link",
    )
    p.add_argument(
        "--search",
        action="append",
        default=[],
        nargs="+",
        help="ensure/add search link",
    )
    p.add_argument(
        "--rss",
        action="append",
        default=[],
        nargs="+",
        help="ensure/add alternate link of type application/rss+xml",
    )
    p.add_argument(
        "--atom",
        action="append",
        default=[],
        nargs="+",
        help="ensure/add alternate link of type application/atom+xml",
    )

    args = p.parse_args(args)
    links = []

    def add_links(links, items, rel, _type=None):
        for href in items:
            d = {}
            d["rel"] = rel
            if _type:
                d["type"] = _type

            if type(href) == list:
                if len(href) == 1:
                    d["href"] = href[0]
                elif len(href) == 2:
                    d["href"] = href[0]
                    d["title"] = href[1]
                elif len(href) == 3:
                    d["href"] = href[0]
                    d["title"] = href[1]
                    d["type"] = href[2]
                else:
                    continue
            else:
                d["href"] = href

            links.append(d)

    for rel in ("stylesheet", "alternate", "next", "prev", "search"):
        add_links(links, getattr(args, rel), rel)
    for item in args.rss:
        add_links(links, item, rel="alternate", _type="application/rss+xml")
    for item in args.atom:
        add_links(links, item, rel="alternate", _type="application/atom+xml")

    # INPUT
    if args.input:
        fin = open(args.input)
    else:
        fin = sys.stdin

    doc = parse(fin, treebuilder="etree", namespaceHTMLElements=False)
    if fin != sys.stdin:
        fin.close()
    html5tidy(
        doc,
        scripts=args.script,
        links=links,
        title=args.title,
        indent=args.indent,
    )

    # OUTPUT
    tmppath = None
    if args.output:
        fout = open(args.output, "w")
    elif args.mogrify:
        tmppath = args.input + ".tmp"
        fout = open(tmppath, "w")
    else:
        fout = sys.stdout

    print(ET.tostring(doc, method=args.method, encoding="unicode"), file=fout)

    if fout != sys.stdout:
        fout.close()

    if tmppath:
        os.rename(args.input, args.input + "~")
        os.rename(tmppath, args.input)


if __name__ == "__main__":
    main(sys.argv)



import json
import os
import re
from argparse import ArgumentParser
from urllib.error import HTTPError, URLError
from urllib.parse import urlencode
from urllib.request import urlopen


def group(items, key=lambda x: x):
    ret = []
    keys = {}
    for item in items:
        k = key(item)
        if k not in keys:
            keys[k] = []
        keys[k].append(item)
    for k in sorted(keys):
        keys[k].sort()
        ret.append(keys[k])
    return ret


def main(args):
    p = ArgumentParser("")
    p.add_argument("input", nargs="+", help="filenames")
    args = p.parse_args(args)

    inputs = args.input
    inputs.sort()

    inputs = [x for x in inputs if not os.path.isdir(x)]

    def base(x):
        return re.sub(r"(\.html)|(\.diff\.html)|(\.meta\.json)|(\.txt)$", "", x)

    # from pprint import pprint
    # pprint()
    gg = group(inputs, base)
    for items in gg:
        itembase = base(items[0])
        try:
            os.mkdir(itembase)
        except OSError:
            pass
        for i in items:
            newloc = os.path.join(itembase, i)
            print("'{0}' => '{1}'".format(i, newloc))
            os.rename(i, newloc)



"""Dumps a CSV of all pads"""

import json
import re
import sys
from argparse import ArgumentParser
from csv import writer
from datetime import datetime
from math import ceil, floor
from urllib.error import HTTPError, URLError
from urllib.parse import urlencode
from urllib.request import urlopen

"""
Dumps a CSV of all pads with columns
padid, groupid, revisions, lastedited, author_ids

    padids have their group name trimmed
    groupid is without (g.  $)
    revisions is an integral number of edits
    lastedited is ISO8601 formatted
    author_ids is a space delimited list of internal author IDs
"""

groupnamepat = re.compile(r"^g\.(\w+)\$")

out = writer(sys.stdout)


def jsonload(url):
    f = urlopen(url)
    data = f.read()
    f.close()
    return json.loads(data)


def main(args):
    p = ArgumentParser("outputs a CSV of information all all pads")
    p.add_argument(
        "--padinfo",
        default=".etherpump/settings.json",
        help="settings, default: .etherdump/settings.json",
    )
    p.add_argument(
        "--zerorevs",
        default=False,
        action="store_true",
        help="include pads with zero revisions, default: False",
    )
    args = p.parse_args(args)

    with open(args.padinfo) as f:
        info = json.load(f)
    apiurl = info.get("apiurl")
    data = {}
    data["apikey"] = info["apikey"]
    requesturl = apiurl + "listAllPads?" + urlencode(data)

    padids = jsonload(requesturl)["data"]["padIDs"]
    padids.sort()
    numpads = len(padids)
    maxmsglen = 0
    count = 0
    out.writerow(("padid", "groupid", "lastedited", "revisions", "author_ids"))
    for i, padid in enumerate(padids):
        p = float(i) / numpads
        percentage = int(floor(p * 100))
        bars = int(ceil(p * 20))
        bar = ("*" * bars) + ("-" * (20 - bars))
        msg = "\r{0} {1}/{2} {3}... ".format(bar, (i + 1), numpads, padid)
        if len(msg) > maxmsglen:
            maxmsglen = len(msg)
        sys.stderr.write("\r{0}".format(" " * maxmsglen))
        sys.stderr.write(msg)
        sys.stderr.flush()
        m = groupnamepat.match(padid)
        if m:
            groupname = m.group(1)
            padidnogroup = padid[m.end() :]
        else:
            groupname = ""
            padidnogroup = padid

        data["padID"] = padid
        revisions = jsonload(apiurl + "getRevisionsCount?" + urlencode(data))[
            "data"
        ]["revisions"]
        if (revisions == 0) and not args.zerorevs:
            continue

        lastedited_raw = jsonload(apiurl + "getLastEdited?" + urlencode(data))[
            "data"
        ]["lastEdited"]
        lastedited_iso = datetime.fromtimestamp(
            int(lastedited_raw) / 1000
        ).isoformat()
        author_ids = jsonload(apiurl + "listAuthorsOfPad?" + urlencode(data))[
            "data"
        ]["authorIDs"]
        author_ids = " ".join(author_ids)
        out.writerow(
            (padidnogroup, groupname, revisions, lastedited_iso, author_ids)
        )
        count += 1

    print("\nWrote {0} rows...".format(count), file=sys.stderr)



"""Extract and output selected fields of metadata"""
import json
import re
import sys
from argparse import ArgumentParser

from .common import *  # noqa

"""
Extract and output selected fields of metadata
"""


def main(args):
    p = ArgumentParser(
        "extract & display meta data from a specific .meta.json file, or for a given padid (nb: it still looks for a .meta.json file)"
    )
    p.add_argument("--path", default=None, help="read from a meta.json file")
    p.add_argument("--padid", default=None, help="read meta for this padid")
    p.add_argument(
        "--format", default="{padid}", help="format str, default: {padid}"
    )
    args = p.parse_args(args)

    path = args.path
    if not path and args.padid:
        path = padpath(args.padid) + ".meta.json"

    if not path:
        print("Must specify either --path or --padid")
        sys.exit(-1)

    with open(path) as f:
        meta = json.load(f)

    formatstr = args.format.decode("utf-8")
    formatstr = re.sub(r"{(\w+)}", r"{0[\1]}", formatstr)
    print(formatstr.format(meta))



"""Check for pads that have changed since last sync (according to .meta.json)"""
import json
import os
import re
import sys
import time
from argparse import ArgumentParser
from datetime import datetime
from fnmatch import fnmatch
from urllib.parse import quote, urlencode
from urllib.request import HTTPError
from xml.etree import ElementTree as ET

import asks
import html5lib
import trio

from etherpump.commands.common import *  # noqa
from etherpump.commands.html5tidy import html5tidy

"""
pull(meta):
        Update meta data files for those that have changed.
        Check for changed pads by looking at revisions & comparing to existing
todo...
use/prefer public interfaces ? (export functions)
"""

# Note(decentral1se): simple globals counting
skipped, saved = 0, 0


async def try_deleting(files):
    for f in files:
        try:
            path = trio.Path(f)
            if os.path.exists(path):
                await path.rmdir()
        except Exception as exception:
            print("PANIC: {}".format(exception))


def build_argument_parser(args):
    parser = ArgumentParser(
        "Check for pads that have changed since last sync (according to .meta.json)"
    )
    parser.add_argument("padid", nargs="*", default=[])
    parser.add_argument(
        "--glob", default=False, help="download pads matching a glob pattern"
    )
    parser.add_argument(
        "--padinfo",
        default=".etherpump/settings.json",
        help="settings, default: .etherpump/settings.json",
    )
    parser.add_argument(
        "--zerorevs",
        default=False,
        action="store_true",
        help="include pads with zero revisions, default: False (i.e. pads with no revisions are skipped)",
    )
    parser.add_argument(
        "--pub",
        default="p",
        help="folder to store files for public pads, default: p",
    )
    parser.add_argument(
        "--group",
        default="g",
        help="folder to store files for group pads, default: g",
    )
    parser.add_argument(
        "--skip",
        default=None,
        type=int,
        help="skip this many items, default: None",
    )
    parser.add_argument(
        "--connection",
        default=50,
        type=int,
        help="number of connections to run concurrently",
    )
    parser.add_argument(
        "--meta",
        default=False,
        action="store_true",
        help="download meta to PADID.meta.json, default: False",
    )
    parser.add_argument(
        "--text",
        default=False,
        action="store_true",
        help="download text to PADID.txt, default: False",
    )
    parser.add_argument(
        "--html",
        default=False,
        action="store_true",
        help="download html to PADID.html, default: False",
    )
    parser.add_argument(
        "--dhtml",
        default=False,
        action="store_true",
        help="download dhtml to PADID.diff.html, default: False",
    )
    parser.add_argument(
        "--all",
        default=False,
        action="store_true",
        help="download all files (meta, text, html, dhtml), default: False",
    )
    parser.add_argument(
        "--folder",
        default=False,
        action="store_true",
        help="dump files in a folder named PADID (meta, text, html, dhtml), default: False",
    )
    parser.add_argument(
        "--output",
        default=False,
        action="store_true",
        help="output changed padids on stdout",
    )
    parser.add_argument(
        "--force",
        default=False,
        action="store_true",
        help="reload, even if revisions count matches previous",
    )
    parser.add_argument(
        "--no-raw-ext",
        default=False,
        action="store_true",
        help="save plain text as padname with no (additional) extension",
    )
    parser.add_argument(
        "--fix-names",
        default=False,
        action="store_true",
        help="normalize padid's (no spaces, special control chars) for use in file names",
    )
    parser.add_argument(
        "--filter-ext", default=None, help="filter pads by extension"
    )
    parser.add_argument(
        "--css",
        default="/styles.css",
        help="add css url to output pages, default: /styles.css",
    )
    parser.add_argument(
        "--script",
        default="/versions.js",
        help="add script url to output pages, default: /versions.js",
    )
    parser.add_argument(
        "--nopublish",
        default="__NOPUBLISH__",
        help="no publish magic word, default: __NOPUBLISH__",
    )
    parser.add_argument(
        "--publish",
        default="__PUBLISH__",
        help="the publish magic word, default: __PUBLISH__",
    )
    parser.add_argument(
        "--publish-opt-in",
        default=False,
        action="store_true",
        help="ensure `--publish` is honoured instead of `--nopublish`",
    )
    parser.add_argument(
        "--magicwords",
        default=False,
        action="store_true",
        help="download html to PADID.magicwords.html",
    )
    return parser


async def get_padids(args, info, data, session):
    if args.padid:
        padids = args.padid
    elif args.glob:
        url = info["localapiurl"] + "listAllPads?" + urlencode(data)
        padids = await agetjson(session, url)
        padids = padids["data"]["padIDs"]
        padids = [x for x in padids if fnmatch(x, args.glob)]
    else:
        url = info["localapiurl"] + "listAllPads?" + urlencode(data)
        padids = await agetjson(session, url)
        padids = padids["data"]["padIDs"]

    padids.sort()
    return padids


async def handle_pad(args, padid, data, info, session):
    global skipped, saved

    raw_ext = ".raw.txt"
    if args.no_raw_ext:
        raw_ext = ""

    data["padID"] = padid
    p = padpath(padid, args.pub, args.group, args.fix_names)
    if args.folder:
        p = os.path.join(p, padid)

    metapath = p + ".meta.json"
    revisions = None
    tries = 1
    skip = False
    padurlbase = re.sub(r"api/1.2.9/$", "p/", info["apiurl"])
    meta = {}

    while True:
        try:
            if os.path.exists(metapath):
                async with await trio.open_file(metapath) as f:
                    contents = await f.read()
                    meta.update(json.loads(contents))
                url = (
                    info["localapiurl"] + "getRevisionsCount?" + urlencode(data)
                )
                response = await agetjson(session, url)
                revisions = response["data"]["revisions"]
                if meta["revisions"] == revisions and not args.force:
                    skip = True
                    reason = "No new revisions, we already have the latest local copy"
                    break

            meta["padid"] = padid
            versions = meta["versions"] = []
            versions.append(
                {"url": padurlbase + quote(padid), "type": "pad", "code": 200,}
            )

            if revisions is None:
                url = (
                    info["localapiurl"] + "getRevisionsCount?" + urlencode(data)
                )
                response = await agetjson(session, url)
                meta["revisions"] = response["data"]["revisions"]
            else:
                meta["revisions"] = revisions

            if (meta["revisions"] == 0) and (not args.zerorevs):
                skip = True
                reason = "0 revisions, this pad was never edited"
                break

            # todo: load more metadata!
            meta["group"], meta["pad"] = splitpadname(padid)
            meta["pathbase"] = p

            url = info["localapiurl"] + "getLastEdited?" + urlencode(data)
            response = await agetjson(session, url)
            meta["lastedited_raw"] = int(response["data"]["lastEdited"])

            meta["lastedited_iso"] = datetime.fromtimestamp(
                int(meta["lastedited_raw"]) / 1000
            ).isoformat()

            url = info["localapiurl"] + "listAuthorsOfPad?" + urlencode(data)
            response = await agetjson(session, url)
            meta["author_ids"] = response["data"]["authorIDs"]

            break
        except HTTPError as e:
            tries += 1
            if tries > 3:
                print(
                    "Too many failures ({0}), skipping".format(padid),
                    file=sys.stderr,
                )
                skip = True
                reason = "PANIC, couldn't download the pad contents"
                break
            else:
                await trio.sleep(1)
        except TypeError as e:
            print(
                "Type Error loading pad {0} (phantom pad?), skipping".format(
                    padid
                ),
                file=sys.stderr,
            )
            skip = True
            reason = "PANIC, couldn't download the pad contents"
            break

    if skip:
        print("[ ] {} (skipped, reason: {})".format(padid, reason))
        skipped += 1
        return

    if args.output:
        print(padid)

    if args.all or (args.meta or args.text or args.html or args.dhtml):
        try:
            path = trio.Path(os.path.split(metapath)[0])
            if not os.path.exists(path):
                await path.mkdir()
        except OSError:
            # Note(decentral1se): the path already exists
            pass

    if args.all or args.text:
        url = info["localapiurl"] + "getText?" + urlencode(data)
        text = await agetjson(session, url)
        ver = {"type": "text"}
        versions.append(ver)
        ver["code"] = text["_code"]

        if text["_code"] == 200:
            text = text["data"]["text"]

            ##########################################
            ## ENFORCE __NOPUBLISH__ MAGIC WORD
            ##########################################
            if args.nopublish in text:
                await try_deleting(
                    (
                        p + raw_ext,
                        p + ".raw.html",
                        p + ".diff.html",
                        p + ".meta.json",
                    )
                )
                print(
                    "[ ] {} (deleted, reason: explicit __NOPUBLISH__)".format(
                        padid
                    )
                )
                skipped += 1
                return False

            ##########################################
            ## ENFORCE __PUBLISH__ MAGIC WORD
            ##########################################
            if args.publish_opt_in and args.publish not in text:
                await try_deleting(
                    (
                        p + raw_ext,
                        p + ".raw.html",
                        p + ".diff.html",
                        p + ".meta.json",
                    )
                )
                print("[ ] {} (deleted, reason: publish opt-out)".format(padid))
                skipped += 1
                return False

            ver["path"] = p + raw_ext
            ver["url"] = quote(ver["path"])
            async with await trio.open_file(ver["path"], "w") as f:
                try:
                    # Note(decentral1se): unicode handling...
                    safe_text = text.encode("utf-8", "replace").decode()
                    await f.write(safe_text)
                except Exception as exception:
                    print("PANIC: {}".format(exception))

            # once the content is settled, compute a hash
            # and link it in the metadata!

            ##########################################
            # INCLUDE __XXX__ MAGIC WORDS
            ##########################################
            if args.all or args.magicwords:
                pattern = r"__[a-zA-Z0-9]+?__"
                all_matches = re.findall(pattern, text)
                magic_words = list(set(all_matches))
                if magic_words:
                    meta["magicwords"] = magic_words

    links = []
    if args.css:
        links.append({"href": args.css, "rel": "stylesheet"})
    # todo, make this process reflect which files actually were made
    versionbaseurl = quote(padid)
    links.append(
        {
            "href": versions[0]["url"],
            "rel": "alternate",
            "type": "text/html",
            "title": "Etherpad",
        }
    )
    if args.all or args.text:
        links.append(
            {
                "href": versionbaseurl + raw_ext,
                "rel": "alternate",
                "type": "text/plain",
                "title": "Plain text",
            }
        )
    if args.all or args.html:
        links.append(
            {
                "href": versionbaseurl + ".raw.html",
                "rel": "alternate",
                "type": "text/html",
                "title": "HTML",
            }
        )
    if args.all or args.dhtml:
        links.append(
            {
                "href": versionbaseurl + ".diff.html",
                "rel": "alternate",
                "type": "text/html",
                "title": "HTML with author colors",
            }
        )
    if args.all or args.meta:
        links.append(
            {
                "href": versionbaseurl + ".meta.json",
                "rel": "alternate",
                "type": "application/json",
                "title": "Meta data",
            }
        )

    if args.all or args.dhtml:
        data["startRev"] = "0"
        url = info["localapiurl"] + "createDiffHTML?" + urlencode(data)
        dhtml = await agetjson(session, url)
        ver = {"type": "diffhtml"}
        versions.append(ver)
        ver["code"] = dhtml["_code"]
        if dhtml["_code"] == 200:
            try:
                dhtml_body = dhtml["data"]["html"]
                ver["path"] = p + ".diff.html"
                ver["url"] = quote(ver["path"])
                doc = html5lib.parse(
                    dhtml_body, treebuilder="etree", namespaceHTMLElements=False
                )
                html5tidy(
                    doc,
                    indent=True,
                    title=padid,
                    scripts=args.script,
                    links=links,
                )
                async with await trio.open_file(ver["path"], "w") as f:
                    output = ET.tostring(doc, method="html", encoding="unicode")
                    await f.write(output)
            except TypeError:
                ver["message"] = dhtml["message"]

    # Process text, html, dhtml, magicwords and all options
    downloaded_html = False
    if args.all or args.html:
        url = info["localapiurl"] + "getHTML?" + urlencode(data)
        html = await agetjson(session, url)
        ver = {"type": "html"}
        versions.append(ver)
        ver["code"] = html["_code"]
        downloaded_html = True

        if html["_code"] == 200:
            try:
                html_body = html["data"]["html"]
                ver["path"] = p + ".raw.html"
                ver["url"] = quote(ver["path"])
                doc = html5lib.parse(
                    html_body, treebuilder="etree", namespaceHTMLElements=False
                )
                html5tidy(
                    doc,
                    indent=True,
                    title=padid,
                    scripts=args.script,
                    links=links,
                )
                async with await trio.open_file(ver["path"], "w") as f:
                    output = ET.tostring(doc, method="html", encoding="unicode")
                    await f.write(output)
            except TypeError:
                ver["message"] = html["message"]

    if args.all or args.magicwords:
        if not downloaded_html:
            html = await agetjson(session, url)
        ver = {"type": "magicwords"}
        versions.append(ver)
        ver["code"] = html["_code"]

        if html["_code"] == 200:
            try:
                html_body = html["data"]["html"]
                ver["path"] = p + ".magicwords.html"
                ver["url"] = quote(ver["path"])
                for magic_word in magic_words:
                    replace_word = (
                        "<span class='highlight'>" + magic_word + "</span>"
                    )
                    if magic_word in html_body:
                        html_body = html_body.replace(magic_word, replace_word)
                doc = html5lib.parse(
                    html_body, treebuilder="etree", namespaceHTMLElements=False
                )
                html5tidy(
                    doc,
                    indent=True,
                    title=padid,
                    scripts=args.script,
                    links=links,
                )
                async with await trio.open_file(ver["path"], "w") as f:
                    output = ET.tostring(doc, method="html", encoding="unicode")
                    await f.write(output)
            except TypeError:
                ver["message"] = html["message"]

    # output meta
    if args.all or args.meta:
        ver = {"type": "meta"}
        versions.append(ver)
        ver["path"] = metapath
        ver["url"] = quote(metapath)
        async with await trio.open_file(metapath, "w") as f:
            await f.write(json.dumps(meta))

    try:
        mwords_msg = ", magic words: {}".format(", ".join(magic_words))
    except UnboundLocalError:
        mwords_msg = ""  # Note(decentral1se): for when magic_words are not counted

    print("[x] {} (saved{})".format(padid, mwords_msg))
    saved += 1
    return


async def handle_pads(args):
    global skipped, saved

    session = asks.Session(connections=args.connection)
    info = loadpadinfo(args.padinfo)
    data = {"apikey": info["apikey"]}

    padids = await get_padids(args, info, data, session)
    if args.skip:
        padids = padids[args.skip : len(padids)]

    print("=" * 79)
    print("Etherpump is warming up the engines ...")
    print("=" * 79)

    start = time.time()
    async with trio.open_nursery() as nursery:
        for padid in padids:
            nursery.start_soon(
                handle_pad, args, padid, data.copy(), info, session
            )
    end = time.time()
    timeit = round(end - start, 2)

    print("=" * 79)
    print(
        "Processed {} :: Skipped {} :: Saved {} :: Time {}s".format(
            len(padids), skipped, saved, timeit
        )
    )
    print("=" * 79)


def main(args):
    p = build_argument_parser(args)
    args = p.parse_args(args)
    trio.run(handle_pads, args)



"""Calls the setHTML API function for the given padid"""

import json
import sys
from argparse import ArgumentParser
from urllib.parse import urlencode
from urllib.request import urlopen

import requests

LIMIT_BYTES = 100 * 1000


def main(args):
    p = ArgumentParser("calls the setHTML API function for the given padid")
    p.add_argument("padid", help="the padid")
    p.add_argument(
        "--html", default=None, help="html, default: read from stdin"
    )
    p.add_argument(
        "--padinfo",
        default=".etherpump/settings.json",
        help="settings, default: .etherdump/settings.json",
    )
    p.add_argument("--showurl", default=False, action="store_true")
    # p.add_argument("--format", default="text", help="output format, can be: text, json; default: text")
    p.add_argument(
        "--create",
        default=False,
        action="store_true",
        help="flag to create pad if necessary",
    )
    p.add_argument(
        "--limit",
        default=False,
        action="store_true",
        help="limit text to 100k (etherpad limit)",
    )
    args = p.parse_args(args)

    with open(args.padinfo) as f:
        info = json.load(f)
    apiurl = info.get("apiurl")
    # apiurl = "{0[protocol]}://{0[hostname]}:{0[port]}{0[apiurl]}{0[apiversion]}/".format(info)
    #    data = {}
    #    data['apikey'] = info['apikey']
    #    data['padID'] = args.padid # is utf-8 encoded

    createPad = False
    if args.create:
        # check if it's in fact necessary
        requesturl = (
            apiurl
            + "getRevisionsCount?"
            + urlencode({"apikey": info["apikey"], "padID": args.padid})
        )
        results = json.load(urlopen(requesturl))
        print(json.dumps(results, indent=2), file=sys.stderr)
        if results["code"] != 0:
            createPad = True

    if args.html:
        html = args.html
    else:
        html = sys.stdin.read()

    params = {}
    params["apikey"] = info["apikey"]
    params["padID"] = args.padid

    if createPad:
        requesturl = apiurl + "createPad"
        if args.showurl:
            print(requesturl)
        results = requests.post(
            requesturl, params=params, data={"text": ""}
        )  # json.load(urlopen(requesturl))
        results = json.loads(results.text)
        print(json.dumps(results, indent=2))

    if len(html) > LIMIT_BYTES and args.limit:
        print("limiting", len(text), LIMIT_BYTES, file=sys.stderr)
        html = html[:LIMIT_BYTES]

    requesturl = apiurl + "setHTML"
    if args.showurl:
        print(requesturl)
    # params['html'] = html
    results = requests.post(
        requesturl,
        params={"apikey": info["apikey"]},
        data={"apikey": info["apikey"], "padID": args.padid, "html": html},
    )  # json.load(urlopen(requesturl))
    results = json.loads(results.text)
    print(json.dumps(results, indent=2))



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['etherpump', 'etherpump.api', 'etherpump.commands']

package_data = \
{'': ['*'], 'etherpump': ['data/templates/*']}

install_requires = \
['asks>=2.4.10,<3.0.0',
 'html5lib>=1.1,<2.0',
 'jinja2>=2.11.2,<3.0.0',
 'pypandoc>=1.5,<2.0',
 'python-dateutil>=2.8.1,<3.0.0',
 'requests>=2.24.0,<3.0.0',
 'trio>=0.17.0,<0.18.0']

entry_points = \
{'console_scripts': ['etherpump = etherpump:main']}

setup_kwargs = {
    'name': 'etherpump',
    'version': '0.0.20',
    'description': 'Pumping text from etherpads into publications',
    'long_description': '# etherpump\n\n[![PyPI version](https://badge.fury.io/py/etherpump.svg)](https://badge.fury.io/py/etherpump)\n[![GPL license](https://img.shields.io/badge/license-GPL-brightgreen.svg)](https://git.vvvvvvaria.org/varia/etherpump/src/branch/master/LICENSE.txt)\n\n_Pumping text from etherpads into publications_\n\nA command-line utility that extends the multi writing and publishing functionalities of the [etherpad](http://etherpad.org/) by exporting the pads in multiple formats.\n\n## Many pads, many networks\n\n_Etherpump_ is a friendly fork of [_etherdump_](https://gitlab.constantvzw.org/aa/etherdump), a command line tool written by [Michael Murtaugh](http://automatist.org/) that converts etherpad pages to files. This fork is made out of curiosities in the tool, a wish to study it and shared sparks of enthusiasm to use it in different situations within Varia.\n\nEtherpump is a stretched version of etherdump. It is a playground in which we would like to add features to the initial tool that diffuse actions of _dumping_ into _pumping_. So most of all, etherpump is a work-in-progress, exploring potential uses of etherpads to edit, structure and publish various types of content.\n\nAdded features are:\n\n- opt-in publishing with the `__PUBLISH__` magic word\n- the `publication` command, that listens to custom magic words such as `__RELEARN__`\n\nSee the [Change log / notes ](#change-log--notes) section for further changes.\n\nEtherpump is a tool that is used from the command line. It pumps all pads of one etherpad installation to a folder, saving them as different text files, such as plain text and HTML. It also creates an index file, that allows one to easily navigate through the list of pads. Etherpump follows a document-driven idea of publishing, which means that it converts pads as database entries into pads as files. This seems to be a redundant act of copying, but is actually an important in-between step that allows for many different publishing projects and experiments.\n\nWe started to get to know etherpump through various editions of Relearn and/or the worksessions organized by Constant. Collaborative writing on an etherpad has been an important ingredient for these situations. The habit of using pads branched into the day-to-day practice of Varia, where we use etherpads for all sorts of things, ranging from organising remote-meetings with 10+ people, to writing and designing PDF documents collaboratively.\n\nAfter installing etherpump on the Varia server, we collectively decided to not want to publish pads by default. Discussions in the group around the use of etherpads, privacy and ideas of what publishing means, led to a need to have etherpump only start the indexing work after it recognizes a `__PUBLISH__` marker on a pad. We decided to work on a `__PUBLISH__ vs. __NOPUBLISH__` branch of etherdump, which we now fork into **etherpump**.\n\n# Change log / notes\n\n**December 2020**\n\nAdded the `--magicwords` flag. Parsing and indexing of magic words is now\nsupported. See [etherpump.vvvvvvaria.org](https://etherpump.vvvvvvaria.org) for\nmore. This is still a work in progress.\n\nChange `--connection` default setting to 50 to avoid overpowering modestly\npowered servers.\n\n**November 2020**\n\nReleasing Etherpump 0.0.18!\n\nHandled a bug that saved the same HTML content in multiple files. Disclaimer: resolved in a hacky way.\n\n---\n\n**October 2020**\n\nUse the more friendly packaging tool [Poetry](https://python-poetry.org/) for publishing.\n\nFurther performance tweaks, informative logging and miscellaneous bug fixing.\n\nDecolonize our Git praxis and use the `main` branch.\n\n---\n\n**January 2020**\n\nAdded experimental [trio](trio.readthedocs.io) and\n[asks](https://asks.readthedocs.io/en/latest/index.html) support for the `pull`\ncommand which enables pads to be processed concurrently. The default\n`--connection` option is set to 20 which may overpower the target server. If in\ndoubt, set this to a lower number (like 5). This functionality is experimental,\nbe cautious and please report bugs!\n\nRemoved fancy progress bars for pulling because concurrent processing makes\nthat hard to track. For now, we simply output whichever padid we\'re finished\nwith.\n\n---\n\n**October 2019**\n\nImprove `etherpump --help` handling to make it easier for new users.\n\nAdded the `python-dateutil` and `pypandoc` dependencies\n\nAdded a fancy progress bar with `tqdm` for long running `etherpump pull --all` calls\n\nStarted with the [experimental library API](#library-api-example).\n\n---\n\n**September 2019**\n\nForking _etherpump_ into _etherpump_.\n\n<https://git.vvvvvvaria.org/varia/etherpump>\n\nMigrating the source code to Python 3.\n\nIntegrate PyPi publishing with setuptools.\n\n---\n\n**May - September 2019**\n\netherpump is used to produce the _Ruminating Relearn_ section of the Network Of One\'s Own 2 (NOOO2) publication.\n\nA new command is added to make a web publication, based on the custom magic word `__RELEARN__`.\n\n---\n\n**June 2019**\n\nMultiple conversations around etherpump emerged during Relearn Curved in Varia, Rotterdam.\n\nIncluding the idea of executable pads (_etherhooks_), custom magic words, a federated snippet protocol (_etherstekje_) and more.\n\n<https://varia.zone/relearn-2019.html>\n\n---\n\n**April 2019**\n\nInstallation of etherpump on the Varia server.\n\n<https://etherpump.vvvvvvaria.org/>\n\n---\n\n**March 2019**\n\nThe `__PUBLISH__ vs. __NOPUBLISH__` was added to the etherpump repository by _decentral1se_.\n\n<https://gitlab.constantvzw.org/aa/etherpump/issues/3>\n\n---\n\nOriginally designed for use at: [Constant](http://etherdump.constantvzw.org/).\n\nMore notes can be found in the [git repository of etherdump](https://gitlab.constantvzw.org/aa/etherdump).\n\n# Install etherpump\n\n`$ pip install etherpump`\n\nEtherpump only supports Python >= 3.6.\n\n## Command-line example\n\n```\n$ mkdir mydump\n$ cd myddump\n$ etherpump init\n```\n\nThe program then interactively asks some questions:\n\n> Please type the URL of the etherpad (e.g. https://pad.vvvvvvaria.org):\n>\n> https://pad.vvvvvvaria.org/\n\nThe APIKEY is the contents of the file APIKEY.txt in the etherpad folder.\n\n> Please paste the APIKEY:\n>\n> xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx\n\nThe settings are placed in a file called `.etherpump/settings.json` and are used (by default) by future commands.\n\n## Common Workflows\n\n### Text+Meta performance wrangling\n\nIf you have a lot of pads, you might want to try the following to speed things\nup. This example is something we do at Varia. Firstly, you download all the\npads text + metadata as the only formats. This is likely what you want when\nyou\'re trying to work directly with the text. You can do that like so:\n\n```bash\n$ etherpump pull --text --meta --publish-opt-in\n```\n\nThe key here is to get the `--meta` so that etherpump is able to read quickly\nskip it on the following run if there are no new revisions. So, in practice,\nyou get a slower first run and faster following runs as more pads are skipped\nfrom actually doing a file system write to save the contents which we already\nhave.\n\n## Library API Example\n\nEtherpump can be used as a library.\n\nAll commands can be imported and run programmatically.\n\n```python\n>>> from etherpump.api import pull\n>>> pull([\'--text\', \'--meta\', \'--publish-opt-in\'])\n```\n\nThere is also a Magic Word interface. It supports the following API:\n\n> magic_word(word, fresh)\n\n- **word**: The magic word to match pad text against (e.g. `__PUB_CLUB__`)\n- **fresh** (default: `True`): Whether or not run a `etherpump pull` each time\n\nHere is an example:\n\n```python\nfrom etherpump.api import magic_word\n\n@magic_word("__PUB_CLUB__", fresh=False)\ndef pub_club_texts(pads):\n    for name in pads:\n        print(pads[name]["txt"])\n\n\npub_club_texts()\n```\n\n`pads` is a dictionary which contains pad names as keys and pad text as values.\nNormally, the `fresh=False` is useful when you\'re hacking away and want to read\npad contents from the local file system and not over the network each time.\n\n## Subcommands\n\nTo see all available subcommands, run:\n\n`$ etherpump --help`\n\nFor help on each individual subcommand, run:\n\n`$ etherpump revisionscount --help`\n\n## Publishing\n\nPlease use ["semver"](https://semver.org/) conventions for versions.\n\nHere are the steps to follow (e.g. for a `0.1.3` release):\n\n- Change the version number in the `etherpump/__init__.py` `__VERSION__` to `0.1.3`\n- Change the version number in the `pyproject.toml` `version` field to `0.1.3`\n- `git add . && git commit -m "Publish new 0.1.3 version" && git tag 0.1.3 && git push --tags`\n- Run `poetry publish --build`\n\nYou should have a [PyPi](https://pypi.org/) account and be added as an owner/maintainer on the [etherpump package](https://pypi.org/project/etherpump/).\n\n## Testing\n\nIt can be quite handy to run a very temporary local Etherpad instance to test against. This is possible with [Docker](https://docs.docker.com/get-docker/).\n\n```bash\n$ docker run -d --name etherpad -p 9001:9001 etherpad/etherpad\n$ docker exec -ti etherpad cat APIKEY.txt;echo\n```\n\nThen you can `etherpump init` to that local Etherpad for experimentation and testing. You use `http://localhost:9001` as the pad URL.\n\nLater on, you can remove the Etherpad with:\n\n```bash\n$ docker rm -f --volumes etherpad\n```\n\n## Maintenance utilities\n\nTools to help things stay tidy over time.\n\n```bash\n$ make\n```\n\nPlease see the following links for further reading:\n\n- [flake8](http://flake8.pycqa.org)\n- [isort](https://isort.readthedocs.io)\n- [black](https://black.readthedocs.io)\n\n### Server Systers Situation\n\n```\n$ sudo -su systers\n$ cd /var/www/etherpump\n$ sh cron.sh\n```\n\nServed from `/etc/nginx/sites-enabled/etherpump.vvvvvvaria.conf`.\n\n## Keeping track of Etherpad-lite\n\n- [Etherpad-lite API documentation](https://etherpad.org/doc/v1.7.5/)\n- [Etherpad-lite releases](https://github.com/ether/etherpad-lite/releases)\n\n# License\n\nGNU AFFERO GENERAL PUBLIC LICENSE, Version 3.\n\nSee [LICENSE](./LICENSE).\n',
    'author': 'Varia, Center for Everyday Technology',
    'author_email': None,
    'maintainer': 'Varia, Center for Everyday Technology',
    'maintainer_email': 'info@varia.zone',
    'url': 'https://git.vvvvvvaria.org/varia/etherpump',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.6,<4.0',
}


setup(**setup_kwargs)



import json
import os
import re
import sys
from html.entities import name2codepoint
from time import sleep
from urllib.parse import quote_plus, unquote_plus
from urllib.request import HTTPError, urlopen

import trio

groupnamepat = re.compile(r"^g\.(\w+)\$")


def splitpadname(padid):
    m = groupnamepat.match(padid)
    if m:
        return (m.group(1), padid[m.end() :])
    else:
        return ("", padid)


def padurl(padid,):
    return padid


def padpath(padid, pub_path="", group_path="", normalize=False):
    g, p = splitpadname(padid)
    p = quote_plus(p)
    if normalize:
        p = p.replace(" ", "_")
        p = p.replace("(", "")
        p = p.replace(")", "")
        p = p.replace("?", "")
        p = p.replace("'", "")
    if g:
        return os.path.join(group_path, g, p)
    else:
        return os.path.join(pub_path, p)


def padpath2id(path):
    dd, p = os.path.split(path)
    gname = dd.split("/")[-1]
    p = unquote_plus(p)
    if gname:
        return "{0}${1}".format(gname, p).decode("utf-8")
    else:
        return p.decode("utf-8")


def getjson(url, max_retry=3, retry_sleep_time=3):
    ret = {}
    ret["_retries"] = 0
    while ret["_retries"] <= max_retry:
        try:
            f = urlopen(url)
            data = f.read()
            data = data.decode("utf-8")
            rurl = f.geturl()
            f.close()
            ret.update(json.loads(data))
            ret["_code"] = f.getcode()
            if rurl != url:
                ret["_url"] = rurl
            return ret
        except ValueError as e:
            url = "http://localhost" + url
        except HTTPError as e:
            print("HTTPError {0}".format(e), file=sys.stderr)
            ret["_code"] = e.code
            ret["_retries"] += 1
            if retry_sleep_time:
                sleep(retry_sleep_time)
    return ret


async def agetjson(session, url):
    """The asynchronous version of getjson."""
    RETRY = 20
    TIMEOUT = 10

    ret = {}
    ret["_retries"] = 0

    try:
        response = await session.get(url, timeout=TIMEOUT, retries=RETRY)
        rurl = response.url
        ret.update(response.json())
        ret["_code"] = response.status_code
        if rurl != url:
            ret["_url"] = rurl
        return ret
    except Exception as e:
        print("Failed to download {}, saw {}".format(url, str(e)))
        return


def loadpadinfo(p):
    with open(p) as f:
        info = json.load(f)
        if "localapiurl" not in info:
            info["localapiurl"] = info.get("apiurl")
    return info


# Python developer Fredrik Lundh (author of elementtree, among other things)
# has such a function on his website, which works with decimal, hex and named
# entities:

##
# Removes HTML or XML character references and entities from a text string.
#
# @param text The HTML (or XML) source text.
# @return The plain text, as a Unicode string, if necessary.
def unescape(text):
    def fixup(m):
        text = m.group(0)
        if text[:2] == "&#":
            # character reference
            try:
                if text[:3] == "&#x":
                    return chr(int(text[3:-1], 16))
                else:
                    return chr(int(text[2:-1]))
            except ValueError:
                pass
        else:
            # named entity
            try:
                text = chr(name2codepoint[text[1:-1]])
            except KeyError:
                pass
        return text  # leave as is

    return re.sub("&#?\w+;", fixup, text)


def istty():
    return sys.stdout.isatty() and os.environ.get("TERM") != "dumb"


def chunks(lst, n):
    for i in range(0, len(lst), n):
        yield lst[i : i + n]



"""Calls the getText API function for the given padid"""

import json
from argparse import ArgumentParser
from urllib.error import HTTPError, URLError
from urllib.parse import urlencode
from urllib.request import urlopen


def main(args):
    p = ArgumentParser("calls the getText API function for the given padid")
    p.add_argument("padid", help="the padid")
    p.add_argument(
        "--padinfo",
        default=".etherpump/settings.json",
        help="settings, default: .etherdump/settings.json",
    )
    p.add_argument("--showurl", default=False, action="store_true")
    p.add_argument(
        "--format",
        default="text",
        help="output format, can be: text, json; default: text",
    )
    args = p.parse_args(args)

    with open(args.padinfo) as f:
        info = json.load(f)
    apiurl = info.get("apiurl")
    data = {}
    data["apikey"] = info["apikey"]
    data["padID"] = args.padid
    requesturl = apiurl + "deletePad?" + urlencode(data)
    if args.showurl:
        print(requesturl)
    else:
        results = json.load(urlopen(requesturl))
        if args.format == "json":
            print(json.dumps(results))
        else:
            if results["data"]:
                print(results["data"]["text"])



"""Calls the getText API function for the given padid"""

import json
import sys
from argparse import ArgumentParser
from urllib.parse import urlencode
from urllib.request import urlopen

import requests

LIMIT_BYTES = 100 * 1000


def main(args):
    p = ArgumentParser("calls the getText API function for the given padid")
    p.add_argument("padid", help="the padid")
    p.add_argument(
        "--text", default=None, help="text, default: read from stdin"
    )
    p.add_argument(
        "--padinfo",
        default=".etherpump/settings.json",
        help="settings, default: .etherdump/settings.json",
    )
    p.add_argument("--showurl", default=False, action="store_true")
    # p.add_argument("--format", default="text", help="output format, can be: text, json; default: text")
    p.add_argument(
        "--create",
        default=False,
        action="store_true",
        help="flag to create pad if necessary",
    )
    p.add_argument(
        "--limit",
        default=False,
        action="store_true",
        help="limit text to 100k (etherpad limit)",
    )
    args = p.parse_args(args)

    with open(args.padinfo) as f:
        info = json.load(f)
    apiurl = info.get("apiurl")
    # apiurl = "{0[protocol]}://{0[hostname]}:{0[port]}{0[apiurl]}{0[apiversion]}/".format(info)
    data = {}
    data["apikey"] = info["apikey"]
    data["padID"] = args.padid  # is utf-8 encoded

    createPad = False
    if args.create:
        requesturl = apiurl + "getRevisionsCount?" + urlencode(data)
        results = json.load(urlopen(requesturl))
        # print (json.dumps(results, indent=2))
        if results["code"] != 0:
            createPad = True

    if args.text:
        text = args.text
    else:
        text = sys.stdin.read()

    if len(text) > LIMIT_BYTES and args.limit:
        print("limiting", len(text), LIMIT_BYTES)
        text = text[:LIMIT_BYTES]

    data["text"] = text

    if createPad:
        requesturl = apiurl + "createPad"
    else:
        requesturl = apiurl + "setText"

    if args.showurl:
        print(requesturl)
    results = requests.post(
        requesturl, params=data
    )  # json.load(urlopen(requesturl))
    results = json.loads(results.text)
    if results["code"] != 0:
        print(
            "setText: ERROR ({0}) on pad {1}: {2}".format(
                results["code"], args.padid, results["message"]
            )
        )
        # json.dumps(results, indent=2)
